/* $Id: derivace.c,v 3.2 1997/08/15 17:16:16 kas Exp $ */

/* Rutiny pro vypocet derivace potencialove funkce */

/*
 *  Programs used in my diploma thesis.
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ff.h"

/*
 * d(rk*(norm(a-b)-req)^2)/d(a1) =
 *    2*(a1-b1)*rk*(norm(a-b)-req)/norm(a-b) 
 */
void diff_bond(ffp, dx, dy, dz, x, y, z, bond)
force_field *ffp;
REAL *dx, *dy, *dz;
REAL *x, *y, *z;
int bond;
{
	int a1 = ffp->bondat1[bond];
	int a2 = ffp->bondat2[bond];

	REAL v12[3], norm, base;

	v12[0] = x[a1]-x[a2]; v12[1] = y[a1]-y[a2]; v12[2] = z[a1]-z[a2]; 
	norm = norm3(v12);
	base = 2 * ffp->rk[ffp->bondtype[bond]]
		* (norm - ffp->req[ffp->bondtype[bond]]) / norm;

	/* We hopefully have common subexpression optimizations ... */
	dx[a1] += v12[0]*base;
	dx[a2] -= v12[0]*base;
	dy[a1] += v12[1]*base;
	dy[a2] -= v12[1]*base;
	dz[a1] += v12[2]*base;
	dz[a2] -= v12[2]*base;
}

void diff_angle(ffp, dx, dy, dz, x, y, z, angle)
force_field *ffp;
REAL *dx, *dy, *dz;
REAL *x, *y, *z;
int angle;
{
	REAL v12[3];
	REAL v32[3];

	int a1 = ffp->angleat1[angle];
	int a2 = ffp->angleat2[angle];
	int a3 = ffp->angleat3[angle];

	REAL norm12, norm32, v12dotv32, theta;

	v12[0] = x[a1]-x[a2]; v12[1] = y[a1]-y[a2]; v12[2] = z[a1]-z[a2];
	v32[0] = x[a3]-x[a2]; v32[1] = y[a3]-y[a2]; v32[2] = z[a3]-z[a2];

	norm12 = norm3(v12);
	norm32 = norm3(v32);
	v12dotv32 = dot3(v12,v32);

	common13 = ffp->tk[ffp->anglenum[angle]] * 2 *
		(acos(v12dotv32/(norm12*norm32))
			- ffp->teq[ffp->anglenum[angle]])
		/ (norm12*norm12*norm32*norm32 - v1dotv2*v1dotv2);

	dx[a1] -= common13 * (v32[0]*norm12*norm32 -
		v12[0]*v1dotv2*norm32/norm12);
	dy[a1] -= common13 * (v32[1]*norm12*norm32 -
		v12[1]*v1dotv2*norm32/norm12);
	dz[a1] -= common13 * (v32[2]*norm12*norm32 -
		v12[2]*v1dotv2*norm32/norm12);
	dx[a3] -= common13 * (v12[0]*norm12*norm32 -
		v32[0]*v1dotv2*norm12/norm32);
	dy[a3] -= common13 * (v12[1]*norm12*norm32 -
		v32[1]*v1dotv2*norm12/norm32);
	dz[a3] -= common13 * (v12[2]*norm12*norm32 -
		v32[2]*v1dotv2*norm12/norm32);

	common2 = ffp->tk
	/* TADY CHCI POKRACOVAT !!! */
}

void diff_dih(ffp, dx, dy, dz, x, y, z, dih)
force_field *ffp;
REAL *dx, *dy, *dz;
REAL *x, *y, *z;
int angle;
{
}

void diff_nb(ffp, dx, dy, dz, x, y, z, a1, a2)
force_field *ffp;
REAL *dx, *dy, *dz;
REAL *x, *y, *z;
int a1, a2;
{
}
