/* $Id: nontrans.c,v 3.12 1997/08/15 17:16:17 kas Exp $ */

/* Vypocty netransformovanych slozek siloveho pole */

/*
 *  Programs used in my diploma thesis.
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <math.h>

#include "ff.h"

/* #define PRINT_DEBUG 1 */
/* #define TOTAL 1 */

REAL compute_bond(ffp, x, y, z, bond)
force_field *ffp;
REAL *x, *y, *z;
int bond;
{
	REAL b;
	int at1, at2;

	at1 = ffp->bondat1[bond];
	at2 = ffp->bondat2[bond];

#ifdef PRINT_DEBUG
	printf("%4d-%4d %7.3f %7.3f ",
		at1+1, at2+1, ffp->rk[ffp->bondnum[bond]],
		ffp->req[ffp->bondnum[bond]]);
	b = sqrt((x[at1] - x[at2])*(x[at1] - x[at2])
		+ (y[at1] - y[at2])*(y[at1] - y[at2])
		+ (z[at1] - z[at2])*(z[at1] - z[at2]));
	printf("%7.3f ", b);
	b -= ffp->req[ffp->bondnum[bond]];
	b = ffp->rk[ffp->bondnum[bond]] * b*b;
	printf ("%7.3f\n", b);
	return b;
#else
	b = sqrt((x[at1] - x[at2])*(x[at1] - x[at2])
		+ (y[at1] - y[at2])*(y[at1] - y[at2])
		+ (z[at1] - z[at2])*(z[at1] - z[at2]))
		- ffp->req[ffp->bondnum[bond]];
	return ffp->rk[ffp->bondnum[bond]] * b*b;
#endif
}

REAL compute_angle(ffp, x, y, z, angle)
force_field *ffp;
REAL *x, *y, *z;
int angle;
{
	REAL a;
	int at1, at2, at3;
	REAL x12[3], x32[3];

	at1 = ffp->angleat1[angle];
	at2 = ffp->angleat2[angle];
	at3 = ffp->angleat3[angle];

#ifdef PRINT_DEBUG
	printf ("%4d-%4d-%4d %7.3f %7.3f ",
		at1+1, at2+1, at3+1, ffp->tk[ffp->anglenum[angle]],
		ffp->teq[ffp->anglenum[angle]]*180.0/M_PI);
#endif
	if (x[at1] - x[at2] == 0.0 && y[at1] - y[at2] == 0.0
		&& z[at1] - z[at2] == 0.0)
	{
		a = -1.0;
		goto computed;
	}

	if (x[at3] - x[at2] == 0.0 && y[at3] - y[at2] == 0.0
		&& z[at3] - z[at2] == 0.0)
	{
		a = -1.0;
		goto computed;
	}

		
	x12[0]= x[at1]-x[at2]; x12[1]= y[at1]-y[at2]; x12[2]= z[at1]-z[at2];
	x32[0]= x[at3]-x[at2]; x32[1]= y[at3]-y[at2]; x32[2]= z[at3]-z[at2];

	a = dot3(x12,x32);
	a /= sqrt(dot3(x12,x12) * dot3(x32,x32));

	/* FIXME: Tady se mozna bude muset zohlednit, jestli ten
	 * rozdil nepreleze ven z intervalu [-pi,pi].
	 */

	a = acos(a);
computed:
#ifdef PRINT_DEBUG
	printf ("%7.3f ", a*180.0/M_PI);
	a -= ffp->teq[ffp->anglenum[angle]];

	printf ("%3.3f\n", a=ffp->tk[ffp->anglenum[angle]]*a*a);
	return a;
#else
	a -= ffp->teq[ffp->anglenum[angle]];

	return ffp->tk[ffp->anglenum[angle]]*a*a;
#endif
}

REAL compute_dih(ffp, x, y, z, dih)
force_field *ffp;
REAL *x, *y, *z;
int dih;
{
	int a1, a2, a3, a4;
	REAL a[3], b[3], c[3];
	REAL total, cos_phi, tmp, n1, n2;
	int n, dihnum;
#ifdef TOTAL
static	REAL vdw14total=0.0, eel14total=0.0, dihtotal=0.0;
#endif
	a1 = ffp->dihat1[dih];
	a2 = ffp->dihat2[dih];
	a3 = ffp->dihat3[dih];
	a4 = ffp->dihat4[dih];
	dihnum = ffp->dihnum[dih];
	/* Normalovy vektor k rovine 1-2-3 */
	a[0] = x[a1] - x[a2];
	a[1] = y[a1] - y[a2];
	a[2] = z[a1] - z[a2];
	b[0] = x[a3] - x[a2];
	b[1] = y[a3] - y[a2];
	b[2] = z[a3] - z[a2];
	c[2] = a[0]*b[1] - a[1]*b[0];	n1 =  c[2]*c[2];
	c[1] = a[2]*b[0] - a[0]*b[2];	n1 += c[1]*c[1];
	c[0] = a[1]*b[2] - a[2]*b[1];	n1 += c[0]*c[0];

	/* Normalovy vektor k rovine 2-3-4 */
	a[0] = x[a4] - x[a3];
	a[1] = y[a4] - y[a3];
	a[2] = z[a4] - z[a3];

	tmp = b[1]*a[0] - b[0]*a[1]; n2  = tmp*tmp; cos_phi  = c[2]*tmp;
	tmp = b[0]*a[2] - b[2]*a[0]; n2 += tmp*tmp; cos_phi += c[1]*tmp;
	tmp = b[2]*a[1] - b[1]*a[2]; n2 += tmp*tmp; cos_phi += c[0]*tmp;

	if (!(n1*=n2))
		return -1.0;

	cos_phi /= sqrt(n1);

	/* Nyni mame v cos_phi cos(<(1-2-3,2-3-4)).
	 * ted z toho potrebujeme dostat cos(Pn*<(1-2-3,2-3-4) - Phase).
	 */

	total = 0.0;
	do {
		REAL temp;
		if ((n=ffp->pn[dihnum]) < 0) {
	/*		printf ("negative:%d\n", n); */
			n = -n;
		}
		switch (n)
		{
		case 1:
			temp = cos_phi;
			break;
		case 2:	/* cos(2x) = 2cos^2(x)-1 */
			temp = 2*cos_phi*cos_phi-1.0;
			break;
		case 3:	/* cos(3x) = 4cos^3(x)-3cos(x) */
			temp = 4*cos_phi*cos_phi*cos_phi-3*cos_phi;
			break;
		case 4: /* cos(4x) = 8cos^4(x)-8cos^2(x)+1 */
			temp = 8*cos_phi*cos_phi*cos_phi*cos_phi
				- 8*cos_phi*cos_phi + 1;
			break;
		default:
			fprintf(stderr, "FATAL: in compute_dih(): Illegal periodicity (%d).\n",n);
			exit(1);
		}

		total += temp = ffp->pk[dihnum]*(1 + ffp->phase[dihnum]*temp);
#ifdef PRINT_DEBUG
	printf ("%4d-%4d-%4d-%4d (%4d) %7.3f %7.3f %7.3f %7.3f\n", a1+1, a2+1,
		a3+1, a4+1,
		dihnum, ffp->phase[dihnum] == 1 ? 0.0 : 180.0,
		ffp->pk[dihnum], acos(cos_phi)*180/M_PI, temp);
#endif
	} while (ffp->pn[dihnum++] < 0);

	if (!ffp->ignored[dih]) {
		total += compute_14(ffp, x,y,z, a1,a4);
#ifdef TOTAL
		/*
		 * Tohle momentln nefunguje -- st tkajc se 1-4
		 * interakc je nutno pesunout do compute_14().
		 */
		dihtotal += total;
		eel14total+=n2/ffp->eel_14;
		vdw14total+=n1/ffp->vdw_14;
		printf("dihtotal=%f, eel14total=%f, vdw14total=%f\n",
			dihtotal, eel14total, vdw14total);
#endif
	}
	return total;
}

/*
 * compute_nb() a compute_14() byly puvodne jedna funkce, ale jednotlive
 * casti se lisily natolik, ze jsem z toho udelal dve funkce.
 */
REAL compute_nb(ffp, x,y,z, a1,a2)
force_field *ffp;
REAL *x, *y, *z;
int a1, a2;
{
	REAL r, vdw, eel;
	int idx;
#ifdef PRINT_DEBUG
	int hbond;
#endif
	idx = ffp->nbnum[ffp->natypes*ffp->atypes[a1]+ffp->atypes[a2]];
#ifdef PRINT_DEBUG
	hbond=(idx<0?1:0);
#endif
	r = (x[a1]-x[a2])*(x[a1]-x[a2])
		+(y[a1]-y[a2])*(y[a1]-y[a2])
		+(z[a1]-z[a2])*(z[a1]-z[a2]);

	/* Momentln je v `r' hodnota r^2. Odmocnme pozdji. */

	/* FIXME: Tady by mlo sp bt r<epsilon pro njak epsilon */
	if (r == 0.0) {
		vdw = eel = 1E+20;
		/*
		 * Nsleduje skok, ale pedpokldm, e pokud nebude
		 * definovno PRINT_DEBUG, tak to kompiltor zoptimalizuje
		 * bez skoku.
		 */
		goto print_and_return;
	}

#if 0
	/* 1991 force field cutoff */
	if (ffp->cutoff_sq > 0.0 && r > ffp->cutoff_sq) {
		vdw = eel = 0.0;
		goto print_and_return;
	}
#endif
	/*
	 * Otestujeme, jestli jde o van der Waalsovskou interakci
	 * (6-12) nebo o vodikovou vazbu (10-12). Pokud je idx < 0,
	 * jde o vodikovou vazbu. Pak je nutne snizit index o dva:
	 * Puvodne bylo pole (Cno ve struct parm) indexovano tak,
	 * ze 1,2,... znamenalo vdw, a -1,-2,... znamenalo vodikovou
	 * vazbu. Protoze vsechny indexove pole snizuji o jednicku
	 * (aby to sedelo na C-style pole), je momentalne v idx
	 * hodnota s vyznamem: 0,1,2,... -- vdw, -2,-3... -- vodikova-2.
	 * FIXME: Existuje nejake elegantnejsi reseni tohoto problemu?
	 */
	if (idx >= 0)
		vdw = ffp->r12[idx]/(r*r*r*r*r*r)
			- ffp->r6[idx]/(r*r*r);
	else
		vdw = ffp->h12[-idx-2]/(r*r*r*r*r*r)
			- ffp->h10[-idx-2]/(r*r*r*r*r);

	r = sqrt(r);
	eel = ffp->charge[a1]*ffp->charge[a2]
		/ ((ffp->diel_lin*r + ffp->diel_const)*r);
print_and_return:
#ifdef PRINT_DEBUG
	printf("%4d-%4d %s(%7.3e,%7.3e): r=%7.3f, vdw=%7.3f, eel=%7.3f\n",
		a1+1,a2+1,
		(hbond?"HBond":"NBond"),
		(hbond?ffp->h12[-idx-2]:ffp->r12[idx]),
		(hbond?ffp->h10[-idx-2]:ffp->r6[idx]), r,
		vdw, eel);
#endif
	return vdw + eel;
}

REAL compute_14(ffp, x,y,z, a1,a2)
force_field *ffp;
REAL *x, *y, *z;
int a1, a2;
{
	REAL r, vdw, eel;
	int idx;

	idx = ffp->nbnum[ffp->natypes*ffp->atypes[a1]+ffp->atypes[a2]];
	/*
	 * Vypocet pro 1-4 interakce je trochu odlisny.
	 * Pocita se zde VdW interakce (12-6) i pro vodikove
	 * atomy. Index se v tomto pripade ziska jako
	 * vetsi_cislo*(vetsi_cislo+1)/2+mensi_cislo.
	 */
	if (idx < 0) {
		int tsmall, tbig, foo;
		tsmall=ffp->atypes[a1];
		tbig=ffp->atypes[a2];
		if (tsmall > tbig) {
			foo = tsmall; tsmall = tbig; tbig = foo;
		}
		idx = (tbig*(tbig+1))/2 + tsmall;
	}

	r = compute_distance_sq(x,y,z, a1, a2);
	/* Momentln je v `r' hodnota r^2. Odmocnme pozdji. */

	if (r == 0.0) {
		vdw = eel = 1E+20;
		goto print_return;
	}

#if 0
	/*
	 * Pedpokldm, e pro 1-4 interakce nem cenu testovat cutoff.
	 */
	/* 1991 force field cutoff */
	if (ffp->cutoff_sq > 0.0 && r > ffp->cutoff_sq) {
		vdw = eel = 0.0;
		goto print_return;
	}
#endif
	vdw = ffp->r12[idx]/(r*r*r*r*r*r)
		- ffp->r6[idx]/(r*r*r);
	vdw /= ffp->vdw_14;

	r = sqrt(r);
	eel = ffp->charge[a1]*ffp->charge[a2]
		/ ((ffp->diel_lin*r + ffp->diel_const)*r);
	eel /= ffp->eel_14;

print_return:
#ifdef PRINT_DEBUG
	printf("%4d-%4d 1-4 int(%7.3e,%7.3e): r=%7.3f, vdw=%7.3f, eel=%7.3f\n",
		a1+1,a2+1,
		ffp->r12[idx],
		ffp->r6[idx], r,
		vdw, eel);
#endif
	return vdw + eel;
}

