/* $Id: printdih.c,v 3.1 1997/03/10 00:03:01 kas Exp $ */

/* Program pro vypis parametru dihedralnich uhlu u molekuly.
 * Vystup tohoto programu ma slouzit jako vstup pro vypocet tabulek
 * ,,vyhlazene`` silove funkce pro torze.
 */

/*
 *  Programs used in my diploma thesis.
 *  Copyright (C) 1995-1997  Jan "Yenya" Kasprzak <kas@fi.muni.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <unistd.h>
#include <getopt.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "ff.h"

#define PRINT_IMPROPER	1	/* Tisknout udaje o nevlastnich dihedralech? */
#define DEFAULT_DIMENSION	13	/* Tabulky po 15 stupnich */
#define ATOM_NAME	"%.2s"
#define RNUM		"% 12.8f"

static char *program_name;

static void usage __P(());
static void die __P((char *msg));
void print_dih_parms __P((FILE *f, force_field *ffp, int adim, int tdim));
	
int main(argc, argv)
int argc;
char **argv;
{
	int adim = 0, tdim = 0;
	force_field *ffp;

	program_name = *argv;

	while(1)
	{
		int c, opt_ind;
		static struct option long_opts[] = {
			{ "help", 0, 0, 'h' },
			{ "version", 0, 0, 'v' },
			{ "dimension", 1, 0, 'd' }, 
			{ "angle-dimension", 1, 0, 'a' },
			{ "torsion-dimension", 1, 0, 't' },
		};

		if ((c=getopt_long(argc, argv, "hvd:a:t:--", long_opts,
			&opt_ind)) == -1)
			break;
		switch(c)
		{
		case 'h':
		case '?':
			usage();
		case 'v':
			printf ("Version: $Id: printdih.c,v 3.1 1997/03/10 00:03:01 kas Exp $\n");
			return 0;
		case 'd':
			if (!adim)
				adim = atoi(optarg);
			else
adim_die:
				die("angle-dimension set more than once!");
			if (!tdim)
				tdim = atoi(optarg);
			else
tdim_die:
				die("torsion-dimension set more than once!");
			break;
		case 'a':
			if (!adim)
				adim = atoi(optarg);
			else
				goto adim_die;
			break;
		case 't':
			if (!tdim)
				tdim = atoi(optarg);
			else
				goto tdim_die;
			break;
		}
	}

	if (optind+1 > argc)
		usage();
	
	if ((ffp = prm2ff(argv[optind])) == NULL)
	{
		perror ("Opening force field file");
		exit (3);
	}

	if (!adim)
		adim = DEFAULT_DIMENSION;
	if (!tdim)
		tdim = DEFAULT_DIMENSION;

	print_dih_parms(stdout, ffp, adim, tdim);

	return 0;
}

/* Funkce vypise parametry dihedralniho uhlu a souradnice prislusnych bodu.
 * Pro kazdy dihedral je vypsano adim*adim*tdim radku, pricemz
 * kazdy radek urcuje souradnice pro ruzne varianty phi, theta1 a theta2.
 * Souradnice bodu jsou generovany z techto uhlu a z rovnovaznych
 * delek vazeb. Body jsou zvoleny v jiste "normalizovane" poloze:
 * Bod 3 je v pocatku souradnic, bod 2 na ose x v zapornem smeru,
 * bod 1 v rovine xy s kladnym y a bod 4 v obecne poloze.
 */
void print_dih_parms(out, ffp, adim, tdim)
FILE *out;
force_field *ffp;
int adim, tdim;
{
	REAL theta1,theta2,phi;
	REAL theta01, theta02, phi0;	/* Equillibrium values */
	REAL l1, l2, l3;
	int d, i, j, k;
	int imp;
	for (d=0; d<ffp->ndihs; d++)
	{
		imp = ffp->improper[d];

		if (imp)
			l1 = ffp->req[find_bond(ffp, ffp->dihat1[d],
				ffp->dihat3[d])];
		else
			l1 = ffp->req[find_bond(ffp, ffp->dihat1[d],
				ffp->dihat2[d])];
		l2 = ffp->req[find_bond(ffp, ffp->dihat2[d], ffp->dihat3[d])];
		l3 = ffp->req[find_bond(ffp, ffp->dihat3[d], ffp->dihat4[d])];

		if (imp)
			theta01 = ffp->teq[find_angle(ffp, ffp->dihat1[d],
				ffp->dihat3[d], ffp->dihat2[d])];
		else
			theta01 = ffp->teq[find_angle(ffp, ffp->dihat1[d],
				ffp->dihat2[d], ffp->dihat3[d])];
		theta02 = ffp->teq[find_angle(ffp, ffp->dihat2[d],
			ffp->dihat3[d], ffp->dihat4[d])];
		phi0 = ffp->phase[ffp->dihnum[d]];

		fprintf(out, ATOM_NAME "-" ATOM_NAME "-" ATOM_NAME "-" ATOM_NAME
			" %2d " RNUM " %d %d %d " RNUM " " RNUM " " RNUM
#ifdef PRINT_IMPROPER
		" %s\n",
#else
		"\n",
#endif
			ffp->atypenames[ffp->dihat1[d]],
			ffp->atypenames[ffp->dihat2[d]],
			ffp->atypenames[ffp->dihat3[d]],
			ffp->atypenames[ffp->dihat4[d]],
			ffp->pn[ffp->dihnum[d]],
			ffp->phase[ffp->dihnum[d]] == -1 ? M_PI : 0.0,
			tdim,adim,adim,
			l1, l2, l3
#ifdef PRINT_IMPROPER
			, imp ? "improper" : "proper");
#else
			);
#endif
#ifdef START_ON_EQUILLIBRIUM_ANGLE
		for (i=0,phi=phi0; i<tdim; i++,phi+=M_PI/(tdim-1))
		for (j=0,theta1=theta01; j<adim; j++,theta1+=M_PI/(adim-1))
		for (k=0,theta2=theta02; k<adim; k++,theta2+=M_PI/(adim-1))
#else
		for (i=0,phi=0.0; i<tdim; i++,phi+=M_PI/(tdim-1))
		for (j=0,theta1=0.0; j<adim; j++,theta1+=M_PI/(adim-1))
		for (k=0,theta2=0.0; k<adim; k++,theta2+=M_PI/(adim-1))
#endif
		{
			fprintf(out, RNUM " " RNUM " " RNUM " ",
				phi, theta1, theta2);
			/* Bod 1 */
			if (imp)
				fprintf(out, RNUM " " RNUM " " RNUM " ",
					-l2+l1*cos(theta1), l1*sin(theta1), 0.0);
			else
				fprintf(out, RNUM " " RNUM " " RNUM " ",
					-l1*cos(theta1), l1*sin(theta1), 0.0);
			/* Bod 2 */
			fprintf(out, RNUM " " RNUM " " RNUM " ",
				-l2, 0.0, 0.0);
			/* Bod 3 */
			fprintf(out, RNUM " " RNUM " " RNUM " ",
				0.0, 0.0, 0.0);
			/* Bod 4 */
			fprintf(out, RNUM " " RNUM " " RNUM "\n",
				-l3*cos(theta2), l3*sin(theta2)*cos(phi),
				l3*sin(theta2)*sin(phi));
		}	/* Tady konci _tri_ for-cykly */
	}
}

static void usage()
{
	printf("Usage: %s [option...] top_file\n"
		"Where option is one or more from:\n"
		"\t-d <number>\tSets the number of points generated in each dimension\n"
		"\t-a <number>\tSets the number of points generated by changing theta1 or theta2\n"
		"\t-t <number>\tSets the number of points generated by changing phi\n"
		"\t-v or --version\tPrints the RCS version of program\n"
		"\t-h or --help\tPrints help\n"
		"\t--dimension\tSame as -d\n"
		"\t--angle-dimension\tSame as -a\n"
		"\t--torsion-dimension\tSame as -t\n"
		, program_name);
	exit (1);
}

static void die(s)
char *s;
{
	fprintf(stderr, "%s\nExiting.\n", s);
	exit(2);
}

